@@ -1,9 +1,10 @@
 package Net::DNS::Resolver::Base;
 #
-# $Id: Base.pm 299 2005-05-27 21:43:34Z olaf $
+# $Id: Base.pm 379 2005-06-14 11:17:50Z olaf $
 #
 
 use strict;
+use bytes;
 
 use vars qw(
 	    $VERSION
@@ -21,7 +22,7 @@ use Net::IP qw(ip_is_ipv4 ip_is_ipv6 ip_normalize);
 use Net::DNS;
 use Net::DNS::Packet;
 
-$VERSION = (qw$LastChangedRevision: 299 $)[1];
+$VERSION = (qw$LastChangedRevision: 379 $)[1];
 
 
 #
@@ -108,6 +109,13 @@ BEGIN {
 		cdflag         => 1,  # this is only used when {dnssec} == 1
 		force_v4       => 0,  # force_v4 is only relevant when we have
                                       # v6 support available
+		ignqrid        => 0,  # normally packets with non-matching ID 
+                                      # or with the qr bit of are thrown away
+			              # in 'ignqrid' these packets are 
+			              # are accepted.
+			              # USE WITH CARE, YOU ARE VULNARABLE TO
+			              # SPOOFING IF SET.
+			              # This is may be a temporary feature
 	);
 	
 	# If we're running under a SOCKSified Perl, use TCP instead of UDP
@@ -143,6 +151,7 @@ my %public_attr = map { $_ => 1 } qw(
 	persistent_tcp
 	persistent_udp
 	dnssec
+	ignqrid
 );
 
 
@@ -277,6 +286,7 @@ sub string {
 
 	my $timeout = defined $self->{'tcp_timeout'} ? $self->{'tcp_timeout'} : 'indefinite';
 	my $hasINET6line= $has_inet6 ?" (IPv6 Transport is available)":" (IPv6 Transport is not available)";
+	my $ignqrid=$self->{'ignqrid'} ? "\n;; ACCEPTING ALL PACKETS (IGNQRID)":"";
 	return <<END;
 ;; RESOLVER state:
 ;;  domain       = $self->{domain}
@@ -290,7 +300,7 @@ sub string {
 ;;  usevc    = $self->{usevc}  stayopen = $self->{stayopen}    igntc = $self->{igntc}
 ;;  defnames = $self->{defnames}  dnsrch   = $self->{dnsrch}
 ;;  recurse  = $self->{recurse}  debug    = $self->{debug}
-;;  force_v4 = $self->{force_v4} $hasINET6line
+;;  force_v4 = $self->{force_v4} $hasINET6line $ignqrid
 END
 
 }
@@ -712,15 +722,15 @@ sub send_udp {
 	
  	if ($self->persistent_udp){
  	    if ($has_inet6 && ! $self->force_v4()){
- 		if ( $self->{'sockets'}[AF_INET6]{'UDP'}) {
- 		    $sock[AF_INET6] = $self->{'sockets'}[AF_INET6]{'UDP'};
- 		    print ";; using persistent AF_INET6 family type socket\n"
+ 		if ( $self->{'sockets'}[AF_INET6()]{'UDP'}) {
+ 		    $sock[AF_INET6()] = $self->{'sockets'}[AF_INET6()]{'UDP'};
+ 		    print ";; using persistent AF_INET6() family type socket\n"
 			if $self->{'debug'};
  		}
  	    }
  	    if ( $self->{'sockets'}[AF_INET]{'UDP'}) {
  		$sock[AF_INET] = $self->{'sockets'}[AF_INET]{'UDP'};
- 		print ";; using persistent AF_INET6 family type socket\n"
+ 		print ";; using persistent AF_INET6() family type socket\n"
  		    if $self->{'debug'};
  	    }
  	    
@@ -736,7 +746,7 @@ sub send_udp {
 		#my $old_wflag = $^W;
 		#$^W = 0;
 		
- 		$sock[AF_INET6] = IO::Socket::INET6->new(
+ 		$sock[AF_INET6()] = IO::Socket::INET6->new(
  							 LocalAddr => $srcaddr,
  							 LocalPort => ($srcport || undef),
  							 Proto     => 'udp',
@@ -764,14 +774,14 @@ sub send_udp {
 
   
  
- 	    unless (defined $sock[AF_INET] || defined $sock[AF_INET6]) {
+ 	    unless (defined $sock[AF_INET] || defined $sock[AF_INET6()]) {
  		$self->errorstring("could not get socket");   #'
  		return;
  	    }
  
  	    
  	    $self->{'sockets'}[AF_INET]{'UDP'} = $sock[AF_INET] if ($self->persistent_udp) && defined( $sock[AF_INET] );
- 	    $self->{'sockets'}[AF_INET6]{'UDP'} = $sock[AF_INET6] if $has_inet6 && ($self->persistent_udp) && defined( $sock[AF_INET6] && ! $self->force_v4() );
+ 	    $self->{'sockets'}[AF_INET6()]{'UDP'} = $sock[AF_INET6()] if $has_inet6 && ($self->persistent_udp) && defined( $sock[AF_INET6()] && ! $self->force_v4() );
  	    
 	    
 	}
@@ -831,7 +841,7 @@ sub send_udp {
 	# We allready tested that one of the two socket exists
 
  	$sel->add($sock[AF_INET]) if defined ($sock[AF_INET]);
- 	$sel->add($sock[AF_INET6]) if $has_inet6 &&  defined ($sock[AF_INET6]) && ! $self->force_v4();
+ 	$sel->add($sock[AF_INET6()]) if $has_inet6 &&  defined ($sock[AF_INET6()]) && ! $self->force_v4();
 	
 
 	# Perform each round of retries.
@@ -862,7 +872,7 @@ sub send_udp {
 			# should skip it.
 			unless (defined ($sock[ $nssockfamily ])){
 			    $self->errorstring("Send error: cannot reach $nsname (" .
-					       ( ($nssockfamily == AF_INET6) ? "IPv6" : "" ).
+					       ( ($nssockfamily == AF_INET6()) ? "IPv6" : "" ).
 					       ( ($nssockfamily == AF_INET) ? "IPv4" : "" ).
 					       ") not available"
 
@@ -901,8 +911,8 @@ sub send_udp {
 				  my ($ans, $err) = Net::DNS::Packet->new(\$buf, $self->{'debug'});
 				  
 				  if (defined $ans) {
-				      next SELECTOR unless $ans->header->qr;
-				      next SELECTOR unless $ans->header->id == $packet->header->id;
+				      next SELECTOR unless ( $ans->header->qr || $self->{'ignqrid'});
+				      next SELECTOR unless  ( ($ans->header->id == $packet->header->id) || $self->{'ignqrid'} );
 				      $self->errorstring($ans->header->rcode);
 				      $ans->answerfrom($self->answerfrom);
 				      $ans->answersize($self->answersize);
@@ -1024,7 +1034,7 @@ sub bgsend {
 							 LocalAddr => $srcaddr,
 							 LocalPort => $srcport,
 					    );
-	} elsif ($sockfamily == AF_INET6 ) {
+	} elsif ($sockfamily == AF_INET6() ) {
 	    $srcaddr="0" if $srcaddr eq "0.0.0.0";  # Otherwise the INET6 socket will just fail
 	    $socket[$sockfamily] = IO::Socket::INET6->new(
 							  Proto => 'udp',
@@ -1041,15 +1051,7 @@ sub bgsend {
 		return;
 	}
 
-
-
-
-
-
-
-
-	print ";; bgsend($ns_address : $dstport)\n" if $self->{'debug'};
-	print ";; remote addr struc: ".unpack("H*", $dst_sockaddr)."\n";
+	print ";; bgsend($ns_address : $dstport)\n" if $self->{'debug'}	;
 
 	foreach my $socket (@socket){
 	    next if !defined $socket;
